@@ -116,22 +116,12 @@ func TestDiscoveredLabelsUpdate(t *testing.T) {
 	}
 	sp.activeTargets = make(map[uint64]*Target)
 	t1 := &Target{
-		discoveredLabels: labels.Labels{
-			labels.Label{
-				Name:  "label",
-				Value: "name",
-			},
-		},
+		discoveredLabels: labels.FromStrings("label", "name"),
 	}
 	sp.activeTargets[t1.hash()] = t1
 
 	t2 := &Target{
-		discoveredLabels: labels.Labels{
-			labels.Label{
-				Name:  "labelNew",
-				Value: "nameNew",
-			},
-		},
+		discoveredLabels: labels.FromStrings("labelNew", "nameNew"),
 	}
 	sp.sync([]*Target{t2})
 
@@ -624,7 +614,7 @@ func TestScrapeLoopStopBeforeRun(t *testing.T) {
 		1,
 		0,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -696,7 +686,7 @@ func TestScrapeLoopStop(t *testing.T) {
 		10*time.Millisecond,
 		time.Hour,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -771,7 +761,7 @@ func TestScrapeLoopRun(t *testing.T) {
 		time.Second,
 		time.Hour,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -826,7 +816,7 @@ func TestScrapeLoopRun(t *testing.T) {
 		time.Second,
 		100*time.Millisecond,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -885,7 +875,7 @@ func TestScrapeLoopForcedErr(t *testing.T) {
 		time.Second,
 		time.Hour,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -943,7 +933,7 @@ func TestScrapeLoopMetadata(t *testing.T) {
 		0,
 		0,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -1000,7 +990,7 @@ func simpleTestScrapeLoop(t testing.TB) (context.Context, *scrapeLoop) {
 		0,
 		0,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -1093,7 +1083,7 @@ func TestScrapeLoopRunCreatesStaleMarkersOnFailedScrape(t *testing.T) {
 		10*time.Millisecond,
 		time.Hour,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -1155,7 +1145,7 @@ func TestScrapeLoopRunCreatesStaleMarkersOnParseFailure(t *testing.T) {
 		10*time.Millisecond,
 		time.Hour,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -1221,7 +1211,7 @@ func TestScrapeLoopCache(t *testing.T) {
 		10*time.Millisecond,
 		time.Hour,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -1303,7 +1293,7 @@ func TestScrapeLoopCacheMemoryExhaustionProtection(t *testing.T) {
 		10*time.Millisecond,
 		time.Hour,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -1417,7 +1407,7 @@ func TestScrapeLoopAppend(t *testing.T) {
 			0,
 			0,
 			false,
-			nil,
+			false,
 			nil,
 			false,
 		)
@@ -1507,7 +1497,7 @@ func TestScrapeLoopAppendForConflictingPrefixedLabels(t *testing.T) {
 					return mutateSampleLabels(l, &Target{labels: labels.FromStrings(tc.targetLabels...)}, false, nil)
 				},
 				nil,
-				func(ctx context.Context) storage.Appender { return app }, nil, 0, true, 0, nil, 0, 0, false, nil, nil, false,
+				func(ctx context.Context) storage.Appender { return app }, nil, 0, true, 0, nil, 0, 0, false, false, nil, false,
 			)
 			slApp := sl.appender(context.Background())
 			_, _, _, err := sl.append(slApp, []byte(tc.exposedLabels), "", time.Date(2000, 1, 1, 1, 0, 0, 0, time.UTC))
@@ -1543,7 +1533,7 @@ func TestScrapeLoopAppendCacheEntryButErrNotFound(t *testing.T) {
 		0,
 		0,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -1601,7 +1591,7 @@ func TestScrapeLoopAppendSampleLimit(t *testing.T) {
 		0,
 		0,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -1678,7 +1668,7 @@ func TestScrapeLoop_ChangingMetricString(t *testing.T) {
 		0,
 		0,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -1726,7 +1716,7 @@ func TestScrapeLoopAppendStaleness(t *testing.T) {
 		0,
 		0,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -1777,7 +1767,7 @@ func TestScrapeLoopAppendNoStalenessIfTimestamp(t *testing.T) {
 		0,
 		0,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -1888,7 +1878,7 @@ metric_total{n="2"} 2 # {t="2"} 2.0 20000
 				0,
 				0,
 				false,
-				nil,
+				false,
 				nil,
 				false,
 			)
@@ -1953,7 +1943,7 @@ func TestScrapeLoopAppendExemplarSeries(t *testing.T) {
 		0,
 		0,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -2005,7 +1995,7 @@ func TestScrapeLoopRunReportsTargetDownOnScrapeError(t *testing.T) {
 		10*time.Millisecond,
 		time.Hour,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -2041,7 +2031,7 @@ func TestScrapeLoopRunReportsTargetDownOnInvalidUTF8(t *testing.T) {
 		10*time.Millisecond,
 		time.Hour,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -2090,7 +2080,7 @@ func TestScrapeLoopAppendGracefullyIfAmendOrOutOfOrderOrOutOfBounds(t *testing.T
 		0,
 		0,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -2135,7 +2125,7 @@ func TestScrapeLoopOutOfBoundsTimeError(t *testing.T) {
 		0,
 		0,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -2392,7 +2382,7 @@ func TestScrapeLoop_RespectTimestamps(t *testing.T) {
 		0,
 		0,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -2433,7 +2423,7 @@ func TestScrapeLoop_DiscardTimestamps(t *testing.T) {
 		0,
 		0,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -2473,7 +2463,7 @@ func TestScrapeLoopDiscardDuplicateLabels(t *testing.T) {
 		0,
 		0,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -2517,7 +2507,7 @@ func TestScrapeLoopDiscardUnnamedMetrics(t *testing.T) {
 		nil, nil,
 		func(l labels.Labels) labels.Labels {
 			if l.Has("drop") {
-				return labels.Labels{}
+				return labels.FromStrings("no", "name") // This label set will trigger an error.
 			}
 			return l
 		},
@@ -2531,7 +2521,7 @@ func TestScrapeLoopDiscardUnnamedMetrics(t *testing.T) {
 		0,
 		0,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -2628,20 +2618,7 @@ func TestReuseScrapeCache(t *testing.T) {
 		}
 		sp, _ = newScrapePool(cfg, app, 0, nil, &Options{})
 		t1    = &Target{
-			discoveredLabels: labels.Labels{
-				labels.Label{
-					Name:  "labelNew",
-					Value: "nameNew",
-				},
-				labels.Label{
-					Name:  "labelNew1",
-					Value: "nameNew1",
-				},
-				labels.Label{
-					Name:  "labelNew2",
-					Value: "nameNew2",
-				},
-			},
+			discoveredLabels: labels.FromStrings("labelNew", "nameNew", "labelNew1", "nameNew1", "labelNew2", "nameNew2"),
 		}
 		proxyURL, _ = url.Parse("http://localhost:2128")
 	)
@@ -2807,7 +2784,7 @@ func TestScrapeAddFast(t *testing.T) {
 		0,
 		0,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -2841,12 +2818,7 @@ func TestReuseCacheRace(t *testing.T) {
 		}
 		sp, _ = newScrapePool(cfg, app, 0, nil, &Options{})
 		t1    = &Target{
-			discoveredLabels: labels.Labels{
-				labels.Label{
-					Name:  "labelNew",
-					Value: "nameNew",
-				},
-			},
+			discoveredLabels: labels.FromStrings("labelNew", "nameNew"),
 		}
 	)
 	defer sp.stop()
@@ -2898,7 +2870,7 @@ func TestScrapeReportSingleAppender(t *testing.T) {
 		10*time.Millisecond,
 		time.Hour,
 		false,
-		nil,
+		false,
 		nil,
 		false,
 	)
@@ -3100,7 +3072,7 @@ func TestScrapeLoopLabelLimit(t *testing.T) {
 			0,
 			0,
 			false,
-			nil,
+			false,
 			nil,
 			false,
 		)
